###############################################
################  RSCRIPT FOR:  ###############
##### "Induced defenses alter the strength ####
### and direction of natural selection on  #### 
### reproductive traits in common milkweed" ###
###############################################
#######  Script author: Ken A. Thompson  ######
########### kthomp1063@gmail.com  #############
###############################################

#Set working directory to path with data csv
setwd("PATH_TO_DATA")

#Read in dataset
dfe <- read.csv("2016-03-01-DFE_Data_CSV.csv")

#Delete plants that were killed during the experiment due to demonic intrusion
#(plants with no final biomass recorded, killed by construction etc.)
dfe <- dfe[!is.na(dfe$Bm),]

#Load packages
library(car)
library(ggplot2)
library(Hmisc)
library(lme4)
library(lmerTest)
library(MuMIn)
library(plyr)
library(psych)


###Relative fitness
#Calculate Relative Fitness and add to dataset (Female)
dfe$RFSeed <- (dfe$Sd_Cnt/mean(dfe[["Sd_Cnt"]]))
#Calculate Relative Fitness and add to dataset (Male)
dfe$RFPol <- (dfe$Poll_Rel/mean(dfe[["Poll_Rel"]]))

######################
###DATA_DIAGNOSTICS###
######################

#Optional: Function to calculate standard error
se <- function(x) sqrt(var(x)/length(x))

#Write diagnostic function to QC variables
diagnostic<-function(x){
  plot(x)
  abline(mean(x),0)
  hist(x)
  qqnorm(x)
  qqline(x,lty=2, col="Red")
  
  skew<-function(x){
    m3<-sum((x-mean(x))^3)/length(x)
    s3<-sqrt(var(x))^3
    m3/s3}
  
  kurtosis<-function(x){
    m4<-sum((x-mean(x))^4)/length(x)
    s4<-var(x)^2
    m4/s4-3}
  
  print(paste("Kurtosis=", kurtosis(x), sep=""))
  print(paste("Skew=", skew(x), sep=""))
  
}

#Check any variables you want, here I am just doing inflorescence number with four types of transformation
diagnostic(dfe$Infl_Num)
diagnostic(sqrt(dfe$Infl_Num))
diagnostic(log1p(dfe$Infl_Num)) #log1p is used rather than 'log' because there are zeroes

diagnostic((dfe$Infl_Num)^2)

#####################
###END_DIAGNOSTICS###
#####################

#Compress floral measurements to two PCs#
#Identify terms for PCA
dfe.flwr.terms <- c("Ptl_Lgth", "Ptl_Wdt", "Spl_Lgth", "Spl_Wdt", "Flwr_Lgth", "Flwr_Wdt")
#Run PCA using
DFE.FLWR.PCA <- principal(dfe[dfe.flwr.terms], nfactors = 2, residuals = FALSE, rotate = "varimax", scores = TRUE, missing = FALSE)
#See eigenvalues
DFE.FLWR.PCA$values
#Create a 2xn matrix of PCA scores
dfe.pca <- DFE.FLWR.PCA$scores
#Attach matrix to main dataframe
dfe <- cbind(dfe, dfe.pca)
#I renamed them with an identifier for length or width
dfe <- rename(dfe, c("RC1"="RC_Wdt", "RC2"="RC_Lgth"))

#############################
###Standardize & transform###
#############################

#I like doing this line-by-line to make the process of
#variable transformation and scaling completely transparent

dfe$Infl_Num.S <- scale(dfe$Infl_Num)
dfe$Flwr_Cnt.S <- scale(dfe$Flwr_Cnt)
dfe$Flwr_Cnt.S <- scale(dfe$Flwr_Cnt)
dfe$Frst_Flwr.S <- scale(log1p(dfe$Frst_Flwr))
dfe$Ptl_Lgth.S <- scale(dfe$Ptl_Lgth)
dfe$Ptl_Wdt.S <- scale(dfe$Ptl_Wdt)
dfe$Spl_Wdt.S <- scale(dfe$Spl_Wdt)
dfe$Spl_Lgth.S <- scale(dfe$Spl_Lgth)
dfe$Flwr_Lgth.S <- scale(dfe$Flwr_Lgth)
dfe$Flwr_Wdt.S <- scale(dfe$Flwr_Wdt)
dfe$RC_Wdt.S <- scale(dfe$RC_Wdt)
dfe$RC_Lgth.S <- scale(dfe$RC_Lgth)
dfe$Poll_Rel.S <- scale(dfe$RC_Lgth)
dfe$Bm.S <- scale(sqrt(dfe$Bm))

#Sometimes PCs act strangely so I make sure R knows they are numbers
dfe$RC_Wdt.S <- as.numeric(dfe$RC_Wdt.S)
dfe$RC_Wdt.S <- as.numeric(dfe$RC_Wdt.S)

########################
###CORREALTION_MATRIX###
########################

#Intial_Rcorr
#Create list of variables
DFE.rcorr.L <- c("Infl_Num.S", "Flwr_Cnt.S", "Ptl_Lgth.S", "Ptl_Wdt.S", "Spl_Lgth.S", "Spl_Wdt.S", "Flwr_Lgth.S", "Flwr_Wdt.S", "Frst_Flwr.S", "Bm.S")
#Cut down dataset
dfe.rcorr.df <- dfe[DFE.rcorr.L]
#convert to matrix
dfe.rcorr.matrix <- as.matrix(dfe.rcorr.df)
#run Rcorr
rcorr(dfe.rcorr.matrix, type="pearson")

#Secondary_Rcorr (with PCs instead of flower measurements)
dfe.rcorr2.L <- c("Infl_Num.S", "Flwr_Cnt.S", "Frst_Flwr.S", "RC_Wdt.S", "RC_Lgth.S", "Bm.S")
dfe.rcorr2.df <- dfe[dfe.rcorr2.L]
dfe.rcorr2.matrix <- as.matrix(dfe.rcorr2.df)
rcorr(dfe.rcorr2.matrix, type="pearson")

########################
###INITIAL_CONDITIONS###
########################

#Check to see if initial conditions of plants differed between treatment levels 
#JA
summary(aov(Init_Lv ~ JA, data = dfe))
summary(aov(Init_Ht ~ JA, data = dfe))
summary(aov(Init_Infl ~ JA, data = dfe))

#Dmg
summary(aov(Init_Lv ~ Dmg, data = dfe))
summary(aov(Init_Ht ~ Dmg, data = dfe))
summary(aov(Init_Infl ~ Dmg, data = dfe))

################
###Mean +- SD###
################
mean(na.omit(dfe$Sd_Cnt))
mean(na.omit(dfe$Poll_Rel))
mean(na.omit(dfe$RC_Wdt))
mean(na.omit(dfe$RC_Lgth))
mean(na.omit(dfe$Frst_Flwr))
mean(na.omit(dfe$Infl_Num))
mean(na.omit(dfe$Flwr_Cnt))
mean(na.omit(dfe$Bm))

sd(na.omit(dfe$Sd_Cnt))
sd(na.omit(dfe$Poll_Rel))
sd(na.omit(dfe$RC_Wdt))
sd(na.omit(dfe$RC_Lgth))
sd(na.omit(dfe$Frst_Flwr))
sd(na.omit(dfe$Infl_Num))
sd(na.omit(dfe$Flwr_Cnt))
sd(na.omit(dfe$Bm))

#########
###CVp###
#########

CVp.Sd_Cnt <- (sqrt(var(na.omit(dfe$Sd_Cnt))) / mean(na.omit(dfe$Sd_Cnt)))*100
CVp.Poll_Rel <- (sqrt(var(na.omit(dfe$Poll_Rel))) / mean(na.omit(dfe$Poll_Rel)))*100

CVp.Infl_Num <- (sqrt(var(dfe$Infl_Num)) / mean(dfe$Infl_Num))*100
CVp.Flwr_Cnt <- (sqrt(var(na.omit(dfe$Flwr_Cnt))) / mean(na.omit(dfe$Flwr_Cnt)))*100
CVp.Frst_Flwr <- (sqrt(var(na.omit(dfe$Frst_Flwr))) / mean(na.omit(dfe$Frst_Flwr)))*100
CVp.RC_Wdt <- (sqrt(var(na.omit(dfe$RC_Wdt))) / mean(na.omit(dfe$RC_Wdt)))*100

CVp.Ptl_Lgth <- (sqrt(var(na.omit(dfe$Ptl_Lgth))) / mean(na.omit(dfe$Ptl_Lgth)))*100
CVp.Ptl_Wdt <- (sqrt(var(na.omit(dfe$Ptl_Wdt))) / mean(na.omit(dfe$Ptl_Wdt)))*100
CVp.Spl_Lgth <- (sqrt(var(na.omit(dfe$Spl_Lgth))) / mean(na.omit(dfe$Spl_Lgth)))*100
CVp.Spl_Wdt <- (sqrt(var(na.omit(dfe$Spl_Wdt))) / mean(na.omit(dfe$Spl_Wdt)))*100
CVp.Flwr_Lgth <- (sqrt(var(na.omit(dfe$Flwr_Lgth))) / mean(na.omit(dfe$Flwr_Lgth)))*100
CVp.Flwr_Wdt <- (sqrt(var(na.omit(dfe$Flwr_Wdt))) / mean(na.omit(dfe$Flwr_Wdt)))*100
CVp.Bm <- (sqrt(var(dfe$Bm)) / mean(dfe$Bm))*100

###########################################
###Treatment effects on traits & fitness###
###########################################

#Damage
summary(lm(Sd_Cnt ~ Dmg, data = dfe))
summary(lm(Poll_Rel ~ Dmg, data = dfe))
summary(lm(Infl_Num.S ~  Dmg, data = dfe))
summary(lm(Flwr_Cnt.S ~ Dmg, data = dfe))
summary(lm(Frst_Flwr.S ~ Dmg, data = dfe))
summary(lm(RC_Wdt.S ~ Dmg, data = dfe))
summary(lm(RC_Lgth.S ~ Dmg, data = dfe))
summary(lm(Bm.S ~ Dmg, data = dfe))

#JA
summary(lm(Sd_Cnt ~ JA, data = dfe))
summary(lm(Poll_Rel ~ JA, data = dfe))
summary(lm(Infl_Num.S ~  JA, data = dfe))
summary(lm(Flwr_Cnt.S ~ JA, data = dfe))
summary(lm(Frst_Flwr.S ~ JA, data = dfe))
summary(lm(RC_Wdt.S ~ JA, data = dfe))
summary(lm(RC_Lgth.S ~ JA, data = dfe))
summary(lm(Bm.S ~ JA, data = dfe))

############################
###MULTIVARIATE_SELECTION###
############################

#Create selection model dataset
#na.omit is neccessary for dredge to work
dfe.MultVar <- na.omit((dfe[, (colnames(dfe) %in% c("Plant_ID", "JA", "Dmg", "RFSeed", "RFPol", "Location", "Infl_Num.S", "Flwr_Cnt.S", "Frst_Flwr.S", "RC_Wdt.S", "RC_Lgth.S", "Bm.S", "Poll_Rel.S"))]))
#Make sure Dmg is numeric and not treated as a factor (i.e., level 5 should be closer to level 6 than level 1)
dfe.MultVar$Dmg <- as.numeric(dfe.MultVar$Dmg)
#Make sure structure of data is as expect
str(dfe.MultVar)

####################
###FEMALE_FITNESS###
####################
#Generate linear selection model to feed into dredge
Fem.MultVar.Seln <- lm(RFSeed ~ (Infl_Num.S + JA + Dmg)^2 + (Flwr_Cnt.S + JA + Dmg)^2 + (Frst_Flwr.S + JA + Dmg)^2 + (RC_Wdt.S + JA + Dmg)^2 + (RC_Lgth.S + JA + Dmg)^2 + (Bm.S + JA + Dmg)^2, data = dfe.MultVar)
#Can analyze this linear model and you can see that all terms are significant, or P < 0.06; however model is far too big for the size of the dataset so I run a dredge analysis to get the most out of the data.
summary(Fem.MultVar.Seln)

#Step 2--Dredge
#Need to set this so dredge analysis works
options(na.action = "na.fail")
#I've constrained the model to explore model space ranging from 7 terms to 14 terms to increase the speed at which it can run. In a full run, all models are between 7 and 14 terms. Remove the final two terms (m.min and m.max) to explore all models.
dredge.Fem.MultVar.Seln <- dredge(Fem.MultVar.Seln, beta = FALSE, evaluate = TRUE, rank = "AIC", m.min = 7, m.max = 14)

#Step 3--Summarize (I interpret results from 'full' and not 'conditional' model)
summary(model.avg(dredge.Fem.MultVar.Seln, subset = delta <=2, beta = TRUE))

#re-enable default setting so other analyses don't crash when there are NAs!
options(na.action = "na.omit")

##################
###MALE_FITNESS###
##################

#Step 1--Make model
DAD.MultVar.Seln <- lm(RFPol ~ (Infl_Num.S + JA + Dmg)^2 + (Flwr_Cnt.S + JA + Dmg)^2 + (Frst_Flwr.S + JA + Dmg)^2 + (RC_Wdt.S + JA + Dmg)^2 + (RC_Lgth.S + JA + Dmg)^2 + (Bm.S + JA + Dmg)^2, data = dfe.MultVar)

#Step 2--Dredge
options(na.action = "na.fail")
dredge.DAD.MultVar.Seln <- dredge(DAD.MultVar.Seln, beta = FALSE, evaluate = TRUE, rank = "AIC", m.min = 4, m.max = 14)

#Step 3--Summarize
summary(model.avg(dredge.DAD.MultVar.Seln, subset = delta <=2, beta = TRUE))

options(na.action = "na.omit")#re-enable default setting

################################
###END MULTIVARIATE_SELECTION###
################################

#Partition dataset based on induction treatment group
dfe.JA <- subset(dfe, JA == "JA")
dfe.CT <- subset(dfe, JA == "Control")

#Difference in Seed Count
mean(dfe.JA$Sd_Cnt)
mean(dfe.CT$Sd_Cnt) 

#How much fewer seeds did JA plants produce?
1- mean(dfe.JA$Sd_Cnt) / mean(dfe.CT$Sd_Cnt) 

#Effect of Damage on biomass.
dfe.Dmg.0 <- subset(dfe, Dmg == "0")
dfe.Dmg.10 <- subset(dfe, Dmg == "10")
dfe.Dmg.20 <- subset(dfe, Dmg == "20")
dfe.Dmg.30 <- subset(dfe, Dmg == "30")
dfe.Dmg.40 <- subset(dfe, Dmg == "40")
dfe.Dmg.50 <- subset(dfe, Dmg == "50")

###########
#RESIDUALS#
###########

#Get residuals from the best fitting model of the data from dredge output.
#Flwr_Cnt
dfe.MultVar$FlwrCt_FitnessResid <- resid(lm(RFSeed ~ JA*Dmg + Infl_Num.S + RC_Wdt.S + Frst_Flwr.S + Bm.S + Infl_Num.S:JA + RC_Wdt.S:JA + Bm.S:JA, data = dfe.MultVar))
dfe.MultVar$FlwrCt_TraitResid <- resid(lm(Flwr_Cnt.S ~ JA*Dmg + Infl_Num.S + RC_Wdt.S + Frst_Flwr.S + Bm.S + Infl_Num.S:JA + RC_Wdt.S:JA + Bm.S:JA, data = dfe.MultVar))

#Infl_Num
dfe.MultVar$InflNum_FitnessResid <- resid(lm(RFSeed ~ JA*Dmg + Flwr_Cnt.S +  RC_Wdt.S + Frst_Flwr.S + Bm.S + Flwr_Cnt.S:JA + RC_Wdt.S:JA + Bm.S:JA, data = dfe.MultVar))
dfe.MultVar$InflNum_TraitResid <- resid(lm(Infl_Num.S ~ JA*Dmg + Flwr_Cnt.S +  RC_Wdt.S + Frst_Flwr.S + Bm.S + Flwr_Cnt.S:JA + RC_Wdt.S:JA + Bm.S:JA, data = dfe.MultVar))

#Bm
dfe.MultVar$Bm_FitnessResid <- resid(lm(RFSeed ~ JA*Dmg + Flwr_Cnt.S +  RC_Wdt.S + Frst_Flwr.S + Infl_Num.S + Flwr_Cnt.S:JA + RC_Wdt.S:JA + Infl_Num.S:JA, data = dfe.MultVar))
dfe.MultVar$Bm_TraitResid <- resid(lm(Bm.S ~ JA*Dmg + Flwr_Cnt.S +  RC_Wdt.S + Frst_Flwr.S + Flwr_Cnt.S + Flwr_Cnt.S:JA + RC_Wdt.S:JA + Flwr_Cnt.S:JA, data = dfe.MultVar))

######################
###DIRECT_SELECTION###
######################
dfe.MultVar.JA <- subset(dfe.MultVar, JA == "JA")
dfe.MultVar.CT <- subset(dfe.MultVar, JA == "Control")

#These analyses are used to calculate selection gradients on plants in a treatment group while controlling for the effects of other traits & treatments on fitness
#INFL_NUM
Infl_Num.DirSel.JA <- lm(InflNum_FitnessResid ~ (InflNum_TraitResid), data = dfe.MultVar.JA)
summary(Infl_Num.DirSel.JA)
Infl_Num.DirSel.CT <- lm(InflNum_FitnessResid ~ (InflNum_TraitResid), data = dfe.MultVar.CT)
summary(Infl_Num.DirSel.CT)

#FLWR_CNT
Flwr_Cnt.DirSel.JA <- lm(FlwrCt_FitnessResid ~ FlwrCt_TraitResid, data = dfe.MultVar.JA)
summary(Flwr_Cnt.DirSel.JA)
Flwr_Cnt.DirSel.CT <- lm(FlwrCt_FitnessResid ~ (FlwrCt_TraitResid), data = dfe.MultVar.CT)
summary(Flwr_Cnt.DirSel.CT)

#Bm
Bm.DirSel.JA <- lm(Bm_FitnessResid ~ (Bm_TraitResid), data = dfe.MultVar.JA)
summary(Bm.DirSel.JA)
Bm.DirSel.CT <- lm(Bm_FitnessResid ~ (Bm_TraitResid), data = dfe.MultVar.CT)
summary(Bm.DirSel.CT)

##############
####Figures###
##############
#THEME
ng1 <- theme(aspect.ratio=1.0,panel.background = element_blank(), 
             panel.grid.major = element_blank(), 
             panel.grid.minor = element_blank(),
             panel.border=element_blank(),
             axis.line = element_line(size=1), 
             axis.line.x = element_line(color="black", size = 1),
             axis.line.y = element_line(color="black", size = 1),
             axis.ticks=element_line(color="black"), 
             axis.text=element_text(color="black"), 
             axis.title=element_text(color="black"), 
             axis.title.y=element_text(vjust=0.2, size=12),
             axis.title.x=element_text(vjust=0.1,size=12),
             axis.text.x=element_text(size=10),
             axis.text.y=element_text(size=10),
             legend.position="none",
             legend.title=element_blank(),
             plot.title = element_blank())

DFE_Fig1B <- ggplot(dfe.MultVar, aes(x = FlwrCt_TraitResid, y = FlwrCt_FitnessResid, group = JA)) +
  labs(x = "Inflorescence size
       (standardized residual)", y = "Relative fitness (residual)") +
  geom_point(aes(shape = factor(JA), fill = factor(JA))) +
  scale_shape_manual(values=c(24, 19)) +
  scale_fill_manual(values=c("white", "black")) +
  geom_smooth(method = "lm", se = F, colour = "black", size = 1.05, fullrange = T, aes(linetype = factor(JA)))  +
  scale_linetype_manual(values=c(5, 1)) +
  ng1
DFE_Fig1B

DFE_Fig1A <- ggplot(dfe.MultVar, aes(x = scale(InflNum_TraitResid), y = InflNum_FitnessResid, group = JA)) +
  labs(x = "Inflorescence number
       (standardized residual)", y = "Relative fitness (residual)") +
  geom_point(aes(shape = factor(JA), fill = factor(JA))) +
  scale_shape_manual(values=c(24, 19)) +
  scale_fill_manual(values=c("white", "black")) +
  geom_smooth(method = "lm", se = F, colour = "black", size = 1.05, fullrange = T, aes(linetype = factor(JA)))  +
  scale_linetype_manual(values=c(5, 1)) +
  ng1
DFE_Fig1A

DFE_FigS2 <- ggplot(dfe.MultVar, aes(x = scale(Bm_TraitResid), y = Bm_FitnessResid, group = JA)) +
  labs(x = "Biomass
       (standardized residual)", y = "Relative fitness (residual)") +
  geom_point(aes(shape = factor(JA), fill = factor(JA))) +
  scale_shape_manual(values=c(24, 19)) +
  scale_fill_manual(values=c("white", "black")) +
  geom_smooth(method = "lm", se = F, colour = "black", size = 1.05, fullrange = T, aes(linetype = factor(JA)))  +
  scale_linetype_manual(values=c(5, 1)) +
  ng1
DFE_FigS2

ggsave(file = "DFE_Fig1A.pdf", plot = DFE_Fig1A, dpi = 600, units = "mm", height = 70, width = 70, bg = "transparent")
ggsave(file = "DFE_Fig1B.pdf", plot = DFE_Fig1B, dpi = 600, units = "mm", height = 70, width = 70, bg = "transparent")

ggsave(file = "DFE_FigS1.pdf", plot = DFE_FigS1, dpi = 600, units = "mm", height = 70, width = 70, bg = "transparent")

summarySE <- function(data=NULL, measurevar, groupvars=NULL, na.rm=FALSE,
                      conf.interval=.95, .drop=TRUE) {
  library(plyr)
  
  # New version of length which can handle NA's: if na.rm==T, don't count them
  length2 <- function (x, na.rm=FALSE) {
    if (na.rm) sum(!is.na(x))
    else       length(x)
  }
  
  # This does the summary. For each group's data frame, return a vector with
  # N, mean, and sd
  datac <- ddply(data, groupvars, .drop=.drop,
                 .fun = function(xx, col) {
                   c(N    = length2(xx[[col]], na.rm=na.rm),
                     mean = mean   (xx[[col]], na.rm=na.rm),
                     sd   = sd     (xx[[col]], na.rm=na.rm)
                   )
                 },
                 measurevar
  )
  
  # Rename the "mean" column    
  datac <- rename(datac, c("mean" = measurevar))
  
  datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean
  
  # Confidence interval multiplier for standard error
  # Calculate t-statistic for confidence interval: 
  # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
  ciMult <- qt(conf.interval/2 + .5, datac$N-1)
  datac$ci <- datac$se * ciMult
  
  return(datac)
}

